import { createElement, Fragment } from 'react';
import { lexer } from 'marked';

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);

    if (enumerableOnly) {
      symbols = symbols.filter(function (sym) {
        return Object.getOwnPropertyDescriptor(object, sym).enumerable;
      });
    }

    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function _typeof(obj) {
  "@babel/helpers - typeof";

  if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
    _typeof = function (obj) {
      return typeof obj;
    };
  } else {
    _typeof = function (obj) {
      return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };
  }

  return _typeof(obj);
}

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

var ReactRenderer = /*#__PURE__*/function () {
  function ReactRenderer() {
    _classCallCheck(this, ReactRenderer);

    this.elementId = 0;
  }

  _createClass(ReactRenderer, [{
    key: "crel",
    value: function crel(el, children, props) {
      var elProps = {
        key: "marked-react-".concat(this.elementId)
      };
      this.elementId += 1;
      return createElement(el, _objectSpread2(_objectSpread2({}, props), elProps), children);
    }
  }, {
    key: "heading",
    value: function heading(text, level) {
      return this.crel("h".concat(level), text);
    }
  }, {
    key: "paragraph",
    value: function paragraph(children) {
      return this.crel('p', children);
    }
  }, {
    key: "text",
    value: function text(_text) {
      return this.crel(Fragment, _text);
    }
  }, {
    key: "link",
    value: function link(href, text, openLinksInNewTab) {
      var target = openLinksInNewTab ? '_blank' : null;
      return this.crel('a', text, {
        href: href,
        target: target
      });
    }
  }, {
    key: "image",
    value: function image(href, text, title) {
      return this.crel('img', null, {
        src: href,
        alt: text,
        title: title
      });
    }
  }, {
    key: "codespan",
    value: function codespan(code, lang) {
      return this.crel('code', code, {
        className: lang || null
      });
    }
  }, {
    key: "code",
    value: function code(_code, lang) {
      return this.crel('pre', this.codespan(_code, lang));
    }
  }, {
    key: "blockquote",
    value: function blockquote(quote) {
      return this.crel('blockquote', quote);
    }
  }, {
    key: "list",
    value: function list(ordered, children) {
      return this.crel(ordered ? 'ol' : 'ul', children);
    }
  }, {
    key: "checkbox",
    value: function checkbox() {
      var checked = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      return this.crel('input', null, {
        type: 'checkbox',
        disabled: true,
        checked: checked
      });
    }
  }, {
    key: "listItem",
    value: function listItem(children) {
      return this.crel('li', children);
    }
  }, {
    key: "table",
    value: function table(children) {
      return this.crel('table', children);
    }
  }, {
    key: "tableHeader",
    value: function tableHeader(children) {
      return this.crel('thead', children);
    }
  }, {
    key: "tableBody",
    value: function tableBody(children) {
      return this.crel('tbody', children);
    }
  }, {
    key: "tableRow",
    value: function tableRow(children) {
      return this.crel('tr', children);
    }
  }, {
    key: "tableCell",
    value: function tableCell(children, flags) {
      var tag = flags.header ? 'th' : 'td';
      return this.crel(tag, children, {
        align: flags.align
      });
    }
  }, {
    key: "strong",
    value: function strong(children) {
      return this.crel('strong', children);
    }
  }, {
    key: "em",
    value: function em(children) {
      return this.crel('em', children);
    }
  }, {
    key: "del",
    value: function del(children) {
      return this.crel('del', children);
    }
  }, {
    key: "html",
    value: function html(_html) {
      return this.crel(Fragment, _html);
    }
  }, {
    key: "hr",
    value: function hr() {
      return this.crel('hr');
    }
  }, {
    key: "br",
    value: function br() {
      return this.crel('br');
    }
  }]);

  return ReactRenderer;
}();

var htmlUnescapes = {
  '&amp;': '&',
  '&lt;': '<',
  '&gt;': '>',
  '&quot;': '"',
  '&#39;': "'"
};
/** Used to match HTML entities and HTML characters. */

var reEscapedHtml = /&(?:amp|lt|gt|quot|#(?:0+)?39);/g;
var reHasEscapedHtml = RegExp(reEscapedHtml.source);
var unescape = function unescape() {
  var string = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  return reHasEscapedHtml.test(string) ? string.replace(reEscapedHtml, function (entity) {
    return htmlUnescapes[entity] || "'";
  }) : string;
};
var joinBase = function joinBase(path, base) {
  if (!base) {
    return path;
  }

  try {
    return new URL(path, base);
  } catch (_unused) {
    return path;
  }
};

var defaultParserOptions = {
  baseURL: null,
  openLinksInNewTab: true,
  langPrefix: 'language-'
};

var ReactParser = /*#__PURE__*/function () {
  function ReactParser() {
    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultParserOptions;

    _classCallCheck(this, ReactParser);

    this.options = options;
    this.renderer = new ReactRenderer();
  }

  _createClass(ReactParser, [{
    key: "parse",
    value: function parse(tokens) {
      var _this = this;

      var langPrefix = this.options.langPrefix;
      return tokens.map(function (token) {
        switch (token.type) {
          case 'space':
            {
              return null;
            }

          case 'hr':
            {
              return _this.renderer.hr();
            }

          case 'heading':
            {
              return _this.renderer.heading(token.text, token.depth);
            }

          case 'paragraph':
            {
              return _this.renderer.paragraph(_this.parseInline(token.tokens));
            }

          case 'text':
            {
              return token.tokens ? _this.parseInline(token.tokens) : token.text;
            }

          case 'blockquote':
            {
              var quote = _this.parse(token.tokens);

              return _this.renderer.blockquote(quote);
            }

          case 'list':
            {
              var children = token.items.map(function (item) {
                var listItemChildren = [];

                if (item.task) {
                  listItemChildren.push(_this.renderer.checkbox(item.checked));
                }

                listItemChildren.push(_this.parse(item.tokens));
                return _this.renderer.listItem(listItemChildren);
              });
              return _this.renderer.list(token.ordered, children);
            }

          case 'code':
            {
              var lang = token.lang ? "".concat(langPrefix).concat(token.lang) : null;
              return _this.renderer.code(token.text, lang);
            }

          case 'html':
            {
              return _this.renderer.html(token.text);
            }

          case 'table':
            {
              var headerCells = token.header.map(function (cell, index) {
                return _this.renderer.tableCell(_this.parseInline(cell.tokens), {
                  header: true,
                  align: token.align[index]
                });
              });

              var headerRow = _this.renderer.tableRow(headerCells);

              var header = _this.renderer.tableHeader(headerRow);

              var bodyChilren = token.rows.map(function (row, index) {
                var rowChildren = row.map(function (cell) {
                  return _this.renderer.tableCell(_this.parseInline(cell.tokens), {
                    header: false,
                    align: token.align[index]
                  });
                });
                return _this.renderer.tableRow(rowChildren);
              });

              var body = _this.renderer.tableBody(bodyChilren);

              return _this.renderer.table([header, body]);
            }

          default:
            {
              console.warn("Token with \"".concat(token.type, "\" type was not found")); // eslint-disable-line no-console

              return null;
            }
        }
      });
    }
  }, {
    key: "parseInline",
    value: function parseInline(tokens) {
      var _this2 = this;

      var _this$options = this.options,
          baseURL = _this$options.baseURL,
          openLinksInNewTab = _this$options.openLinksInNewTab;
      return tokens.map(function (token) {
        switch (token.type) {
          case 'text':
            {
              return _this2.renderer.text(unescape(token.text));
            }

          case 'escape':
            {
              return _this2.renderer.text(token.text);
            }

          case 'strong':
            {
              return _this2.renderer.strong(_this2.parseInline(token.tokens));
            }

          case 'em':
            {
              return _this2.renderer.em(_this2.parseInline(token.tokens));
            }

          case 'del':
            {
              return _this2.renderer.del(_this2.parseInline(token.tokens));
            }

          case 'codespan':
            {
              return _this2.renderer.codespan(unescape(token.text));
            }

          case 'br':
            {
              return _this2.renderer.br();
            }

          case 'link':
            {
              var href = joinBase(token.href, baseURL);
              return _this2.renderer.link(href, _this2.parseInline(token.tokens), openLinksInNewTab);
            }

          case 'image':
            {
              var _href = joinBase(token.href, baseURL);

              return _this2.renderer.image(_href, token.text, token.title);
            }

          case 'html':
            {
              return _this2.renderer.html(token.text);
            }

          default:
            console.warn("Token with \"".concat(token.type, "\" type was not found")); // eslint-disable-line no-console

            return null;
        }
      });
    }
  }]);

  return ReactParser;
}();

var defaultLexerOptions = {
  breaks: false,
  gfm: true
};

var defaultOptions = _objectSpread2(_objectSpread2({}, defaultLexerOptions), defaultParserOptions);

var validateComponentProps = function validateComponentProps(props) {
  if (props.value && typeof props.value !== 'string') {
    throw new TypeError("[marked-react]: Expected value to be of type string but got ".concat(_typeof(props.value)));
  }

  if (props.children && typeof props.children !== 'string') {
    throw new TypeError("[marked-react]: Expected children to be of type string but got ".concat(_typeof(props.children)));
  }
};

var Markdown = function Markdown(props) {
  var _ref, _props$value;

  validateComponentProps(props); // assign default options

  var options = _objectSpread2(_objectSpread2({}, defaultOptions), props.options); // lexer options


  var lexerOptions = {
    breaks: options.breaks,
    gfm: options.gfm
  }; // convert input markdown into tokens

  var tokens = lexer((_ref = (_props$value = props.value) !== null && _props$value !== void 0 ? _props$value : props.children) !== null && _ref !== void 0 ? _ref : '', lexerOptions); // parser options

  var parserOptions = {
    baseURL: options.baseURL,
    openLinksInNewTab: options.openLinksInNewTab,
    langPrefix: options.langPrefix
  };
  var children = new ReactParser(parserOptions).parse(tokens);
  return createElement(Fragment, null, children);
};

Markdown.defaultProps = {
  options: defaultOptions
};

export { Markdown as default };
//# sourceMappingURL=marked-react.js.map
